 /***********************************************************************
 *
 Sample Code Disclaimer

 Copyright  2003 PalmSource, Inc. or its subsidiaries.  All
 rights reserved.

 You may incorporate this sample code (the "Code") into your applications
 for Palm OS(R) platform products and may use the Code to develop
 such applications without restriction.  The Code is provided to you on
 an "AS IS" basis and the responsibility for its operation is 100% yours.
 PALM, INC. AND ITS SUBSIDIARIES (COLLECTIVELY, "PALM") DISCLAIM
 ALL WARRANTIES, TERMS AND CONDITIONS WITH RESPECT TO THE CODE, EXPRESS,
 IMPLIED, STATUTORY OR OTHERWISE, INCLUDING WARRANTIES, TERMS OR
 CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
 NONINFRINGEMENT AND SATISFACTORY QUALITY.  You are not permitted to
 redistribute the Code on a stand-alone basis and you may only
 redistribute the Code 473in object code form as incorporated into your
 applications.  TO THE FULL EXTENT ALLOWED BY LAW, PALM ALSO EXCLUDES ANY
 LIABILITY, WHETHER BASED IN CONTRACT OR TORT (INCLUDING NEGLIGENCE), FOR
 INCIDENTAL, CONSEQUENTIAL, INDIRECT, SPECIAL OR PUNITIVE DAMAGES OF ANY
 KIND, OR FOR LOSS OF REVENUE OR PROFITS, LOSS OF BUSINESS, LOSS OF
 INFORMATION OR DATA, OR OTHER FINANCIAL LOSS ARISING OUT OF OR IN
 CONNECTION WITH THE USE OR PERFORMANCE OF THE CODE.  The Code is subject
 to Restricted Rights for U.S. government users and export regulations.

 SAMPLE NAME:		ReceivePoll

 FILE:     			ReceivePoll.c

 DESCRIPTION:		Sample for polling for serial data while doing other things.
 					PalOS 3.5 or higher.		

 Version:			1.0
 
 *
 **********************************************************************/

#include <PalmOS.h>
#include "../RSC/StarterRsc.h"
#include <ExpansionMgr.h>


/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;

static Boolean AppHandleEvent(EventPtr eventP);

/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'dsrp'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01

//max time for the next alarm to trigger.
#define nextAlarmTimeInSeconds	3		//seconds.

// Define the minimum OS version we support (2.0 for now).
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)
#define kPalmOS10Version	sysMakeROMVersion(1,0,0,sysROMStageRelease,0)

//define the version of our features.
#define ourPortRefFeature 0


//function prototypes.
static void ReadData();
static void * GetObjectPtr(UInt16 objectID);
static Boolean CheckData();
static void SetAlarmProc(UInt32 nextAlarm);

Boolean 	gReadyP;					// port ready flag
Boolean 	IsDataReady;				// data ready flag
Char 		buffer[1024];				// serial buffer.

static int iExpansionCards=0;
Char szExpCards[10][80];


//callback to handle the alarm trigger.
static void MyAlarmProc (AlmProcCmdEnum almProcCmd, SysAlarmTriggeredParamType* paramP);
static void MyAlarmProc (AlmProcCmdEnum almProcCmd, SysAlarmTriggeredParamType* paramP)
{
	//check the data
	if(CheckData())
		ReadData();
		
	//We are done reading the data, set the next alarm.
	SetAlarmProc(2);					//if we get here that means we have a dialog, use
										//shorter timeouts.
}

//set an alarm callback to perform some special task with or without user interaction.
static void SetAlarmProc(UInt32 nextAlarm)
{
	Err 	err;
	UInt32	curTime, nextAlarmTime;
	UInt32 	gP = 0;
	
	if(nextAlarm == 0)
	{
		//Cancel any pending alarm we might have.
		AlmSetProcAlarm(&MyAlarmProc, (UInt32)gP, NULL);
		return;
	}	
	
	// Get the current time
	curTime = TimGetSeconds();
	
	//Set the next alarm trigger time.
	nextAlarmTime = curTime + nextAlarm;
	
	//Set a procedural alarm
	err = AlmSetProcAlarm(&MyAlarmProc, (UInt32)gP, nextAlarmTime);
	if(err)
		ErrAlert(err);

}

//Event check and dispatch routine. This routine also takes care of disabling
//any alarms set by the app when the device goes to sleep.
static Boolean CheckAndDispatchEvents()
{
	EventType 	event;
	Err 		error;
	
	//Check for UI events and process them.
	EvtGetEvent(&event, 1);
	if(event.eType == appStopEvent)
	{
		//requeue so that the app can exit.
		EvtAddEventToQueue(&event);
		return false;			//break;		
	}	
	
	//check to see if the device is going to sleep. If true, disable the alarm.
	if(event.data.keyDown.chr == vchrPowerOff)
		SetAlarmProc(0); 		//we are going to sleep stop the alarm.
	
	//dispatch events.
	if (! SysHandleEvent(&event))
		if (! MenuHandleEvent(0, &event, &error))
			if (! AppHandleEvent(&event))
				FrmDispatchEvent(&event);	
	
	return true;
}

//Just for fun animate a rectangle so that we can see the Alarm is getting called while 
//other work being done. The data should only be displayed from a UI thread so there might
//be some delay in updating the UI!!
static void Animate()
{
	RectangleType 	rect;
	int 			i=0,j=0;
	
	rect.topLeft.x = 0;
	rect.topLeft.y = 140;
	rect.extent.x  = 160;
	rect.extent.y  = 158;

	//Draw a solid Rectangle with round conners
	WinDrawRectangle(&rect, simple3DFrame);
	
	while(1)
	{	
		//check for events, return false if appStop event is detected.
		if(!CheckAndDispatchEvents())
			break;	
		
		//Delay for visual.
		SysTaskDelay(5);
		
		//just repaint with different colors.
		WinPaintRectangle(&rect, simple3DFrame);
		
		//Change the forground color for fun!!
		WinSetForeColor(i);
		
		j++;
		//delay the color changing.
		if(j==12)
		{	
			i += 5;
			j=0;				//reset the delay.
			if(i == 255)
				i=0;			//reset the color.
		}					
		
	}
	
	//Erase the rectangle
	WinEraseRectangle(&rect, simple3DFrame);	
}

static void AppendToField(const char *s)
{
	FormPtr form = FrmGetFormPtr(MainForm);
	FieldPtr resultsField = (FieldPtr) FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainTextField));
	MemHandle h = FldGetTextHandle(resultsField);
	Err	err;
	Int16 incomingLen = StrLen(s);
	UInt32 startOffset = 0;
	Char *p;
	UInt16	visible;
	UInt16	length;
	
	if (h) {
		FldSetTextHandle(resultsField, NULL);
		startOffset = MemHandleSize(h) - 1;
		err = MemHandleResize(h, MemHandleSize(h) + incomingLen);
		ErrFatalDisplayIf(err, "Can't resize handle");
	} else {
		h = MemHandleNew(incomingLen + 1);
		ErrFatalDisplayIf(!h, "Can't allocate handle");
	}
	p = (Char *) MemHandleLock(h);
	StrCopy(p + startOffset, s);
	MemHandleUnlock(h);
	FldSetTextHandle(resultsField, h);
	
		// can't try to set scroll position unless this is top form
	if (FrmGetActiveForm() == form) {
		length = FldGetTextHeight(resultsField);
		visible = FldGetVisibleLines(resultsField);
		FldSetScrollPosition (resultsField, (UInt16) MemHandleSize(h) - 1);

		//FldDrawField(resultsField); 
	}

}

static Boolean CheckData();
static Boolean CheckData()
{
	Err 	err = 0;
   	UInt32 	count;
   	UInt32 	portRef;

	//The alarm is set to make sure we get CheckData called even if we have some
	//dialog eating up our events. This guarantees that we check for data in all 
	//cases. The best time interval is left for apps to decide based on their needs.
	//You CAN'T SET TIME OF ONE SECOND. 
	
	//This call takes some extra prosessor cycles
	//so apps can ignore resetting alarm and just handle the callback.
	SetAlarmProc(nextAlarmTimeInSeconds);
	
	err = FtrGet(appFileCreator, ourPortRefFeature,&portRef);
	if(err)
	{
		ErrAlert(err);
		return false;
	}

	SrmReceiveCheck(portRef, &count);
	if(count > 0)
	{
		return true;
	}	
	else
		return false;
}

//Routine to read the data from the serial receive queue.
static void ReadData();
static void ReadData()
{
	Err 	err = 0;
   	UInt32 	count;
   	UInt8* 	tempP;
   	UInt32 	portRef;
   	char 	tmp[10];

	//get our port reference
	err = FtrGet(appFileCreator, ourPortRefFeature,&portRef);
	if(err)
	{
		ErrAlert(err);
		return;
	}
	
	while (true) 
	{
		// Get the data
		err = SrmReceiveWindowOpen(portRef, &tempP, &count);
		if (err)
			SrmClearErr(portRef);
		if (!count)
			break;
		
		if(tempP)
		{
			//I am using the event pump to print the data I receive
			//the real app don't have to do this as it has other meaningful
			//things to do with the data!!
			
			//test code to monitor data reads. Must be removed in the real app!
			StrPrintF(tmp, "%ld", count);
			WinDrawChars(tmp, StrLen(tmp), 80, 1);
			
			//using global is not a good idea!!
			MemMove(&buffer, tempP, count);
			buffer[count] = 0;
			IsDataReady = true;					//set the data ready flag.
		}
		
		// Let the serial manager know that we're done with the data
		SrmReceiveWindowClose(portRef, count);
	}
}

//Main routine to open the port and set the polling.
static void SetRcvPoll();
static void SetRcvPoll()
{
	Err 	err;
	UInt32 	flagSettings;
	UInt16 	dataLen;
	UInt16  portRef;
	UInt16 slotRefNum;
	UInt32 slotIterator = expIteratorStart; 
	ExpCardInfoType sExpInfo;
	int i;
	
	
	flagSettings = srmDefaultSettings; 
	dataLen = sizeof(flagSettings);
	
	err = SrmOpen(0x8000, 9600, &portRef);
	if(!err)
	{
		//save the port reference so that app can use it later.
		FtrSet(appFileCreator, ourPortRefFeature, (UInt32) portRef);
			
		SrmControl(portRef, srmCtlSetFlags, &flagSettings, &dataLen);
			
		//to work around a flow control problem on the Lubock board, we have
		//to reset the receive buffer.
		SrmSetReceiveBuffer(portRef, 0L,0L);
			
		//send some characters to the other device.
		SrmSend(portRef, "Open ", 5, &err);
		SrmSendWait(portRef);
		
		gReadyP = true;					//port is ready.
#if 0
		for (i=0; i<10; i++)
			memset(szExpCards[i], 0, sizeof(szExpCards[0]));
  
		while (slotIterator != expIteratorStop) 
		{ 
		   // Get the slotRefNum for the next slot 
		   err = ExpSlotEnumerate(&slotRefNum, &slotIterator); 
		   if(err == errNone) 
		   { 
		      // perform slot-specific processing here 
				ExpCardInfo(slotRefNum, &sExpInfo);
//				WinDrawChars(sExpInfo.manufacturerStr, StrLen(sExpInfo.manufacturerStr), 80, 1);
				
				strcpy(szExpCards[iExpansionCards],sExpInfo.manufacturerStr);
				iExpansionCards++;
		   } else { 
		      // handle error... possibly by 
		      // breaking out of the loop 
		//      AppendToField("Error getting ExpCardInfo\r\n");
		   }
		} 

		if (iExpansionCards==0)
		{
			iExpansionCards=1;
			strcpy(szExpCards[0], "No expansion card detected\r\n");
		}
			
#endif		
		return;	
	}
	else
		ErrAlert(err);


Exit:
	if(portRef)
		SrmClose(portRef);
}		

/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion <= kPalmOS10Version)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (command)
		{
		
		case MainOptionsAboutStarterApp:
			MenuEraseStatus(0);					// Clear the menu status from the display.
			frmP = FrmInitForm (AboutForm);
			FrmDoDialog (frmP);					// Display the About Box.
			FrmDeleteForm (frmP);
			handled = true;
			break;

		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	FormPtr frmP;
	int i;
	
	//check for data and read it if any!
	if(gReadyP)
		if(CheckData())
			ReadData();	
#if 0	
	//update the text if we received something.
	if (iExpansionCards)
	{
		for (i=0; i<iExpansionCards; i++)
			AppendToField(szExpCards[i]);
	}
	iExpansionCards=0;
#endif	
	if(IsDataReady)
	{
		AppendToField(buffer);
		IsDataReady = false;
	}
	
	//regular event processing.
	switch (eventP->eType) 
		{
		
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			SetRcvPoll();
			Animate();
			handled = true;
			break;
			
		case frmUpdateEvent:
			// To do any custom drawing here, first call FrmDrawForm(), then do your
			// drawing, and then set handled to true.
			frmP = FrmGetActiveForm();
			FrmDrawForm(frmP);
			handled = true;
			break;
			
		case keyDownEvent:
		if (eventP->data.keyDown.chr == vchrPageDown || eventP->data.keyDown.chr == vchrPageUp) {
			FieldPtr field;
			
			frmP = FrmGetFormPtr(MainForm);
			field = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, MainTextField));
			FldScrollField(field, 1, (eventP->data.keyDown.chr == vchrPageUp) ? winUp : winDown);
			handled = true;
		}
		break;	

		default:
			break;
		
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do { //setting a wait time guarantees an event or nilEvent in that time.
		EvtGetEvent(&event, /*evtWaitForever*/ 1);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	StarterPreferenceType prefs;
	UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
	gReadyP = false;
	IsDataReady = false;
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	StarterPreferenceType prefs;
	UInt32 portRef;

	// Write the saved preferences / saved-state information.  This data 
	// will saved during a HotSync backup.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	gReadyP = false;
	IsDataReady = false;
	FtrGet(appFileCreator, ourPortRefFeature,&portRef);
	if(portRef)
	{
		AlmSetAlarm(0, DmFindDatabase(0, "ReceivePoll"), 0, 0, false);
		//SrmSetReceiveBuffer(portRef, 0L, 0);
   		//MemPtrFree(newRcvBuffer);  
		SetAlarmProc(0); 		//stop the alarm.
		SrmClose(portRef);
	}
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    StarterPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;
		
		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
